 /*
 *  file      : fileselect.c
 *  project   : zoomc
 *  with      : Gtk-2
 *
 *  copyright : (C) 2010 - 2011 by Claude Bulin
 *
 *  ZOOMC - GTK+ implementation of the GNU shell command
 *  GNU General Public License
 *
 *  This file is part of ZOOMC.
 * 
 *  ZOOMC is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  at your option) any later version.
 * 
 *  ZOOMC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with ZOOMC.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../libzoom/lib.h"
#endif

#include "global.h"
#include "poche.h"
#include "fileselect.h"




/*
*---------------------------------------------------------------------------
* DEFINITIONS
*---------------------------------------------------------------------------
*/

typedef enum {

	_REP_SIMPLE_,
	_REP_MULTIPLE_,
	_FILE_SIMPLE_,
	_FILE_MULTIPLE_,
	_FILE_IMG_SIMPLE_,
	_FILE_IMG_MULTIPLE_

} TYPE_SELECT;


typedef struct {
	gchar		 *Title;
	TYPE_SELECT	  TypeSelect;
	GtkWidget	 *AdrFileChooser;
	void		  (*Call_Func) (GtkButton *button, gpointer user_data);
	void		  (*Func_Extern) (gchar *string);
} SELECT;


typedef struct {

	gboolean		 BoolInitDatas;
	TYPE_FILESELECTION	 TypeFileSelection;
	
} VAR_FILESELECT;

VAR_FILESELECT	VarFileSelect = { FALSE };


/*
*---------------------------------------------------------------------------
* PROTOTYPES
*---------------------------------------------------------------------------
*/

void ON_button_PATH_IMPORT_IMAGES_clicked (GtkButton *button, gpointer user_data);


/*
*---------------------------------------------------------------------------
* DATAS
*---------------------------------------------------------------------------
*/

SELECT Select [ ] =
{

{NULL,	_FILE_IMG_MULTIPLE_,	NULL, ON_button_PATH_IMPORT_IMAGES_clicked,			NULL},
 
{NULL, -1}
};



// CHOIX D UN FICHIER IMAGE
// 
void ON_button_PATH_IMPORT_IMAGES_clicked (GtkButton *button, gpointer user_data)
{
/*
	gint	Choice = GPOINTER_TO_INT(user_data);
	PRINT_FUNC_LF();
	gtk_widget_destroy(Select[ Choice ].AdrFileChooser);
	Select[ Choice ].AdrFileChooser = NULL;
*/
	gint	Choice = GPOINTER_TO_INT(user_data);
	gchar	*Path = NULL;
	GSList	*List = NULL;

	Path = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (Select[ Choice ].AdrFileChooser));
	if (Select[ Choice ].Func_Extern) (*Select[ Choice ].Func_Extern) (Path);
	g_free (Path);
	Path = NULL;
	List = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (Select[ Choice ].AdrFileChooser));
	gtk_widget_destroy(Select[ Choice ].AdrFileChooser);
	Select[ Choice ].AdrFileChooser = NULL;
	
	pochedir_add_img_file (List);
	
	g_slist_free (List);
	List = NULL;
}


/*
*---------------------------------------------------------------------------
* GESTION FILESELECT
*---------------------------------------------------------------------------
*/

// Demande de destruction de la fenetre
// 
gboolean fileselect_on_filechooserdialog_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	gint    Choice = GPOINTER_TO_INT(user_data);

	/* PRINT_FUNC_LF(); */
	/*
	g_print ("user_data = %d\n", (gint)user_data);
	g_print ("Choice    = %d\n", Choice);
	*/
	gtk_widget_destroy(Select[ Choice ].AdrFileChooser);
	Select[ Choice ].AdrFileChooser = NULL;
	return FALSE;
}
// Demande de destruction de la fenetre
// 
gboolean fileselect_on_filechooserdialog_destroy_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	gint    Choice = GPOINTER_TO_INT(user_data);

	/* PRINT_FUNC_LF(); */
	/*
	g_print ("user_data = %d\n", (gint)user_data);
	g_print ("Choice    = %d\n", Choice);
	*/
	gtk_widget_destroy(Select[ Choice ].AdrFileChooser);
	Select[ Choice ].AdrFileChooser = NULL;
	return FALSE;
}
// 
// 
void fileselect_on_button_cancel_clicked (GtkButton *button, gpointer user_data)
{
	gint    Choice = GPOINTER_TO_INT(user_data);

	/* PRINT_FUNC_LF(); */
	/*
	g_print ("user_data = %d\n", (gint)user_data);
	g_print ("Choice    = %d\n", Choice);
	*/
	gtk_widget_destroy(Select[ Choice ].AdrFileChooser);
	Select[ Choice ].AdrFileChooser = NULL;
}
// 
// 
void fileselect_on_button_ok_clicked (GtkButton *button, gpointer user_data)
{
	gint    Choice = GPOINTER_TO_INT(user_data);

	/* PRINT_FUNC_LF(); */
	/*
	g_print ("user_data = %d\n", (gint)user_data);
	g_print ("Choice    = %d\n", Choice);
	*/
	gtk_widget_destroy(Select[ Choice ].AdrFileChooser);
	Select[ Choice ].AdrFileChooser = NULL;
}
// http://forum.gtk-fr.org/viewtopic.php?pid=22743
// 
void chooserDialog_preview_update (GtkFileChooser *filechooser, gpointer data)
{
	/*-- WIDGETS --*/
	GtkWidget	*preview = NULL;
	GdkPixbuf	*pixbuf = NULL;
	gchar		*filename = NULL;
	gboolean	have_preview;

	preview = GTK_WIDGET (data);
	filename = gtk_file_chooser_get_preview_filename(filechooser);

	/*
	Lors de l'ouverture d'un repertoire, filename vaut toujours NULL,
	(du moins sous windows).
	Si le test n'est pas fait,
	la seule concequence est un WARNING en console.
	*/
	if (filename != NULL) {
		/*Creation du pixbud de l'image*/
		pixbuf = gdk_pixbuf_new_from_file_at_size(filename, 150, 150, NULL);
		/*Definition de l'etat d'affichage de previsualisation*/
		have_preview = (pixbuf != NULL);

		g_free (filename);
		filename = NULL;

		/*Creation de l'image*/
		gtk_image_set_from_pixbuf (GTK_IMAGE(preview), pixbuf);
		/*if (pixbuf) gdk_pixbuf_unref (pixbuf);*/
		if (pixbuf) g_object_unref(pixbuf);
		
		gtk_file_chooser_set_preview_widget_active(filechooser, have_preview);
	}
}
// 
// 
void fileselect_remove_comment (void)
{
	gint index;
		
	for (index = 0; index < _NB_PATH_; index ++) {
		g_free (Select[ index ].Title);
		Select[ index ].Title = NULL;
	}
}
// 
// 
void fileselect_init_comment (void)
{
	Select[ _PATH_IMPORT_IMAGES_ ].Title		= g_strdup (_("Importer des images pour la pochette"));
}
// Demande de creation de la fenetre de selection de fichier(s) ou de repertoire
// 
void fileselect_create (TYPE_FILESELECTION Choice, gchar *Path, void *Func_Extern)
{
	GtkWidget *filechooserdialog;
	GtkWidget *dialog_vbox1;
	GtkWidget *dialog_action_area1;
	GtkWidget *button_cancel;
	GtkWidget *button_ok;
	GtkWidget *preview;

	if (VarFileSelect.BoolInitDatas == FALSE) {
		fileselect_init_comment ();
		VarFileSelect.BoolInitDatas = TRUE;
	}
	
	if (_NB_PATH_ <= 0) return;
	if (Choice < 0 || Choice > _NB_PATH_) return;
	
	VarFileSelect.TypeFileSelection = Choice;
	
	if (Choice < _PATH_IMPORT_IMAGES_ || Choice > _NB_PATH_) return;
	filechooserdialog = Select[ Choice ].AdrFileChooser;

	if (filechooserdialog != NULL) {
		gdk_window_raise (filechooserdialog->window);
		return;
	}

	/* CORRECTIF DU 26 01 2006 */
	Select[ Choice ].Func_Extern = Func_Extern;

	/* CORRECTIF DU 28 05 2006 */
	g_utf8_validate (Select[ Choice ].Title, -1, NULL);
	if (NULL == Select[ Choice ].Title) {
		PRINT("\tTITLE == NULL");
		return;
	}

	/*
	typedef enum
	{
	  GTK_FILE_CHOOSER_ACTION_OPEN,
	  GTK_FILE_CHOOSER_ACTION_SAVE,
	  GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
	  GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER
	} GtkFileChooserAction;
	*/
	if (Select[ Choice ].TypeSelect == _REP_SIMPLE_ || Select[ Choice ].TypeSelect == _REP_MULTIPLE_) {
		/* selection de directory */
		filechooserdialog = gtk_file_chooser_dialog_new (Select[ Choice ].Title,
								  NULL,
								  GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
								  /* |
								  GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER,
								  */
								  NULL,
								  NULL);
	}
	else if (Select[ Choice ].TypeSelect == _FILE_SIMPLE_ || Select[ Choice ].TypeSelect == _FILE_MULTIPLE_) {
		/* selection de fichiers(s) */
		filechooserdialog = gtk_file_chooser_dialog_new (Select[ Choice ].Title,
								  NULL,
								  GTK_FILE_CHOOSER_ACTION_OPEN,
								  NULL,
								  NULL);
	}
	else if (Select[ Choice ].TypeSelect == _FILE_IMG_SIMPLE_ || Select[ Choice ].TypeSelect == _FILE_IMG_MULTIPLE_) {
		/* selection de fichiers images avec previsualisation */
		filechooserdialog = gtk_file_chooser_dialog_new (Select[ Choice ].Title,
								  NULL,
								  GTK_FILE_CHOOSER_ACTION_OPEN,
								  /*GTK_STOCK_CANCEL,
								  GTK_RESPONSE_CANCEL,
								  GTK_STOCK_OPEN,
								  GTK_RESPONSE_ACCEPT,*/
								  NULL,
								  NULL);
		preview = gtk_image_new ();
		gtk_widget_set_usize(GTK_WIDGET(preview), 200, 200);
		gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(filechooserdialog),  GTK_WIDGET(preview));
		gtk_file_chooser_set_preview_widget_active (GTK_FILE_CHOOSER (filechooserdialog), TRUE);
		g_signal_connect(GTK_FILE_CHOOSER(filechooserdialog), "update-preview", G_CALLBACK (chooserDialog_preview_update), preview);
	}


	if (Select[ Choice ].TypeSelect == _REP_MULTIPLE_ ||
		Select[ Choice ].TypeSelect == _FILE_MULTIPLE_ ||
		Select[ Choice ].TypeSelect == _FILE_IMG_MULTIPLE_) {
		/* selection 'multiple' */
		g_object_set (filechooserdialog, "select-multiple", TRUE, NULL);
		gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (filechooserdialog), TRUE);
	}

	gtk_window_set_position (GTK_WINDOW (filechooserdialog), GTK_WIN_POS_CENTER);

	gtk_window_set_transient_for (GTK_WINDOW(filechooserdialog), GTK_WINDOW(WindMain));
	gtk_window_set_modal (GTK_WINDOW (filechooserdialog), TRUE);

	gtk_file_chooser_set_current_folder (
					GTK_FILE_CHOOSER (filechooserdialog),
					Path);

	Select[ Choice ].AdrFileChooser = filechooserdialog;

	gtk_window_set_type_hint (GTK_WINDOW (filechooserdialog), GDK_WINDOW_TYPE_HINT_DIALOG);

	dialog_vbox1 = GTK_DIALOG (filechooserdialog)->vbox;
	gtk_widget_show (dialog_vbox1);

	dialog_action_area1 = GTK_DIALOG (filechooserdialog)->action_area;
	gtk_widget_show (dialog_action_area1);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

	button_cancel = gtk_button_new_from_stock ("gtk-cancel");
	gtk_widget_show (button_cancel);
	gtk_dialog_add_action_widget (GTK_DIALOG (filechooserdialog), button_cancel, GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS (button_cancel, GTK_CAN_DEFAULT);

	button_ok = gtk_button_new_from_stock ("gtk-open");
	gtk_widget_show (button_ok);
	gtk_dialog_add_action_widget (GTK_DIALOG (filechooserdialog), button_ok, GTK_RESPONSE_OK);
	GTK_WIDGET_SET_FLAGS (button_ok, GTK_CAN_DEFAULT);

	g_signal_connect ((gpointer) filechooserdialog, "delete_event", G_CALLBACK (fileselect_on_filechooserdialog_delete_event), (gpointer)Choice);
	g_signal_connect ((gpointer) filechooserdialog, "destroy_event", G_CALLBACK (fileselect_on_filechooserdialog_destroy_event), (gpointer)Choice);
	g_signal_connect ((gpointer) button_cancel, "clicked", G_CALLBACK (fileselect_on_button_cancel_clicked), (gpointer)Choice);
	g_signal_connect ((gpointer) button_ok, "clicked", G_CALLBACK (Select[ Choice ].Call_Func), (gpointer)Choice);

	/* Store pointers to all widgets, for use by lookup_widget().
	*/
	GLADE_HOOKUP_OBJECT_NO_REF (filechooserdialog, filechooserdialog, "filechooserdialog");
	GLADE_HOOKUP_OBJECT_NO_REF (filechooserdialog, dialog_vbox1, "dialog_vbox1");
	GLADE_HOOKUP_OBJECT_NO_REF (filechooserdialog, dialog_action_area1, "dialog_action_area1");
	GLADE_HOOKUP_OBJECT (filechooserdialog, button_cancel, "button_cancel");
	GLADE_HOOKUP_OBJECT (filechooserdialog, button_ok, "button_ok");

	gtk_widget_grab_default (button_ok);
  	gtk_widget_show (filechooserdialog);
}

