 /*
 *  file      : poche.c
 *  project   : zoomc
 *  with      : Gtk-2
 *
 *  copyright : (C) 2010 - 2011 by Claude Bulin
 *
 *  ZOOMC - GTK+ implementation of the GNU shell command
 *  GNU General Public License
 *
 *  This file is part of ZOOMC.
 * 
 *  ZOOMC is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  at your option) any later version.
 * 
 *  ZOOMC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with ZOOMC.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <glib/gprintf.h>
#include <string.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>	// M_PI
#include <cairo.h>
#include <cairo-pdf.h>
#include <cairo-ps.h>
#include <cairo-xlib.h>
#include <X11/Xlib.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../libzoom/lib.h"
#endif

#include "global.h"
#include "cursor.h"
#include "dragNdrop.h"
#include "fileselect.h"
#include "poche.h"



VIEW view = {
	NULL,			// *AdrDrawingarea
	FALSE,			// BoolScaleAdjust
	NULL,			// *image
	1.0,			// scale
	0.0, 0.0,		// x0,  y0
	0, 0,			// image_width,  image_height
	NULL,			// *ListImage
	FALSE,			// BoolEventButtonPress
	NULL,			// *GetImage
	IMG_NONE,		// ImgCorner
	NULL,			// *Adr_viewport_image_preview
	NULL,			// *Adr_table
	NULL			// *glist
	};


gdouble oldX = -1.0;
gdouble oldY = -1.0;



// 
// 
gboolean poche_is_erase_active( void )
{
	GList	*List = NULL;
        IMAGE	*Image = NULL;
        gboolean	BoolEraseActive = FALSE;
        
	List = g_list_first( view.ListImage );
	while( List ) {
		if( NULL != (Image = (IMAGE *)List->data ) && TRUE == Image->BoolStructActived ) {
			return( TRUE );
		}
		List = g_list_next( List );
	}
	return( BoolEraseActive );
}
// 
// 
void poche_set_flag_buttons (void)
{
	gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("button_erase")), poche_is_erase_active() );
	gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("button_load_file")), TRUE );
	gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("button_save")), TRUE );
	gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("button_moins")), TRUE );
	gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("button_normal")), TRUE );
	gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("button_plus")), TRUE );
	gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("button_ajuster")), TRUE );
}
// 
// 
void poche_print_zoom_changed( gdouble p_zoom_scale )
{
	gchar	*Str = NULL;
	
	Str = g_strdup_printf( "  %3.0f%%  ", 100.0 * p_zoom_scale );
	gtk_label_set_text( GTK_LABEL(GLADE_GET_OBJECT("label_percent")), Str );
	g_free( Str );
	Str = NULL;
}
// 
// 
void poche_set_size_request( void )
{
	gtk_widget_set_size_request( view.AdrDrawingarea, (gint)((gdouble)view.image_width * view.scale), (gint)((gdouble)view.image_height * view.scale) );
}
// 
// 
gdouble poche_get_scale_adjust( void )
{
	gdouble	return_scale = 1.0;
	
	if( TRUE == view.BoolScaleAdjust ) {
		gdouble	scale_width = 1.0;
		gdouble	scale_height = 1.0;
		gdouble	new_scale = 1.0;
		
		// LA DIMENSION DE L IMAGE DOIT ETRE INFERIEURE A LA TAILLE INITIALE DE LA FENETRE
		// DANS MON CAS:
		// 	TAILLE IMAGE:   324 x 420
		// 	TAILLE FENETRE: 400 x 500
		// SCALE A 100 %  RESOUD DONC LE PROBLEME
		if( view.scale > 1.0 ) {
			view.scale = 1.0;
			poche_set_size_request();
		}
		scale_width = (gdouble)view.AdrDrawingarea->allocation.width / (gdouble)view.image_width;
		scale_height = (gdouble)view.AdrDrawingarea->allocation.height / (gdouble)view.image_height;
		new_scale = MIN(scale_width, scale_height);
		return_scale = new_scale;
	}
	else {
		return_scale = view.scale;
	}
	poche_print_zoom_changed( return_scale );
	return( return_scale );
}
// 
// 
void poche_draw_object( cairo_t *cr, IMAGE *p_Image )
{
	if( NULL != p_Image->Pixbuf )
		g_object_unref( p_Image->Pixbuf );
	p_Image->Pixbuf  = gdk_pixbuf_scale_simple( p_Image->PixbufOriginal, p_Image->image_width, p_Image->image_height, 2 );	// GDK_INTERP_NEAREST
	
	p_Image->image_width  = gdk_pixbuf_get_width (p_Image->Pixbuf);
	p_Image->image_height = gdk_pixbuf_get_height (p_Image->Pixbuf);
	
	// cairo_save (cr);
	
	cairo_scale( cr, view.scale, view.scale );
	cairo_translate( cr, view.x0 + p_Image->x0, view.y0 + p_Image->y0 );
	gdk_cairo_set_source_pixbuf (cr, (GdkPixbuf *)p_Image->Pixbuf, 0., 0. );
	cairo_paint( cr );

	// cairo_restore (cr);
}
// 
// 
void poche_get_handle_move( IMAGE *p_Image, gboolean p_BoolTestCursor )
{
	if( TRUE == p_BoolTestCursor ) {
		
		// POUR LES TESTS CURSEUR SOURIS
		
		view.HandleMove.zone [ IMG_HAUT_GAUCHE ] .x      =  p_Image->x0 -4;
		view.HandleMove.zone [ IMG_HAUT_GAUCHE ] .y      =  p_Image->y0 -4;
		view.HandleMove.zone [ IMG_HAUT_GAUCHE ] .width  =  9;
		view.HandleMove.zone [ IMG_HAUT_GAUCHE ] .height =  9;
	
		view.HandleMove.zone [ IMG_HAUT ] .x             = (p_Image->x0 + (p_Image->image_width / 2)) -4;
		view.HandleMove.zone [ IMG_HAUT ] .y             =  p_Image->y0 -4;
		view.HandleMove.zone [ IMG_HAUT ] .width         =  9;
		view.HandleMove.zone [ IMG_HAUT ] .height        =  9;
		
		view.HandleMove.zone [ IMG_HAUT_DROIT ] .x       = (p_Image->x0 + p_Image->image_width) -4;
		view.HandleMove.zone [ IMG_HAUT_DROIT ] .y       =  p_Image->y0 -4;
		view.HandleMove.zone [ IMG_HAUT_DROIT ] .width   =  9;
		view.HandleMove.zone [ IMG_HAUT_DROIT ] .height  =  9;
		
		view.HandleMove.zone [ IMG_DROIT ] .x            = (p_Image->x0 + p_Image->image_width) -4;
		view.HandleMove.zone [ IMG_DROIT ] .y            = (p_Image->y0 + (p_Image->image_height / 2)) -4;
		view.HandleMove.zone [ IMG_DROIT ] .width        =  9;
		view.HandleMove.zone [ IMG_DROIT ] .height       =  9;
		
		view.HandleMove.zone [ IMG_BAS_DROIT ] .x        = (p_Image->x0 + p_Image->image_width) -4;
		view.HandleMove.zone [ IMG_BAS_DROIT ] .y        = (p_Image->y0 + p_Image->image_height) -4;
		view.HandleMove.zone [ IMG_BAS_DROIT ] .width    =  9;
		view.HandleMove.zone [ IMG_BAS_DROIT ] .height   =  9;
		
		view.HandleMove.zone [ IMG_BAS ] .x              = (p_Image->x0 + (p_Image->image_width / 2)) -4;
		view.HandleMove.zone [ IMG_BAS ] .y              =  (p_Image->y0 + p_Image->image_height) -4;
		view.HandleMove.zone [ IMG_BAS ] .width          =  9;
		view.HandleMove.zone [ IMG_BAS ] .height         =  9;
		
		view.HandleMove.zone [ IMG_BAS_GAUCHE ] .x       =  p_Image->x0  -4;
		view.HandleMove.zone [ IMG_BAS_GAUCHE ] .y       = (p_Image->y0 + p_Image->image_height) -4;
		view.HandleMove.zone [ IMG_BAS_GAUCHE ] .width   =  9;
		view.HandleMove.zone [ IMG_BAS_GAUCHE ] .height  =  9;
		
		view.HandleMove.zone [ IMG_GAUCHE ] .x           =  p_Image->x0  -4;
		view.HandleMove.zone [ IMG_GAUCHE ] .y           = (p_Image->y0 + (p_Image->image_height / 2)) -4;
		view.HandleMove.zone [ IMG_GAUCHE ] .width       =  9;
		view.HandleMove.zone [ IMG_GAUCHE ] .height      =  9;
		
		view.HandleMove.zone [ IMG_MOVE ] .x             =  p_Image->x0;
		view.HandleMove.zone [ IMG_MOVE ] .y             =  p_Image->y0;
		view.HandleMove.zone [ IMG_MOVE ] .width         =  p_Image->image_width;
		view.HandleMove.zone [ IMG_MOVE ] .height        =  p_Image->image_height;
	}
	else {
		// POUR L AFFICHAGE DES RECTANGLES DE SAISIES
		
		view.HandleMove.zone [ IMG_HAUT_GAUCHE ] .x      = -4;
		view.HandleMove.zone [ IMG_HAUT_GAUCHE ] .y      = -4;
		view.HandleMove.zone [ IMG_HAUT_GAUCHE ] .width  =  9;
		view.HandleMove.zone [ IMG_HAUT_GAUCHE ] .height =  9;
	
		view.HandleMove.zone [ IMG_HAUT ] .x             = (p_Image->image_width / 2) -4;
		view.HandleMove.zone [ IMG_HAUT ] .y             = -4;
		view.HandleMove.zone [ IMG_HAUT ] .width         =  9;
		view.HandleMove.zone [ IMG_HAUT ] .height        =  9;
		
		view.HandleMove.zone [ IMG_HAUT_DROIT ] .x       =  p_Image->image_width -4;
		view.HandleMove.zone [ IMG_HAUT_DROIT ] .y       = -4;
		view.HandleMove.zone [ IMG_HAUT_DROIT ] .width   =  9;
		view.HandleMove.zone [ IMG_HAUT_DROIT ] .height  =  9;
		
		view.HandleMove.zone [ IMG_DROIT ] .x            =  p_Image->image_width -4;
		view.HandleMove.zone [ IMG_DROIT ] .y            = (p_Image->image_height / 2) -4;
		view.HandleMove.zone [ IMG_DROIT ] .width        =  9;
		view.HandleMove.zone [ IMG_DROIT ] .height       =  9;
		
		view.HandleMove.zone [ IMG_BAS_DROIT ] .x        = p_Image->image_width -4;
		view.HandleMove.zone [ IMG_BAS_DROIT ] .y        = p_Image->image_height -4;
		view.HandleMove.zone [ IMG_BAS_DROIT ] .width    =  9;
		view.HandleMove.zone [ IMG_BAS_DROIT ] .height   =  9;
		
		view.HandleMove.zone [ IMG_BAS ] .x              = (p_Image->image_width / 2) -4;
		view.HandleMove.zone [ IMG_BAS ] .y              =  p_Image->image_height -4;
		view.HandleMove.zone [ IMG_BAS ] .width          =  9;
		view.HandleMove.zone [ IMG_BAS ] .height         =  9;
		
		view.HandleMove.zone [ IMG_BAS_GAUCHE ] .x       = -4;
		view.HandleMove.zone [ IMG_BAS_GAUCHE ] .y       =  p_Image->image_height -4;
		view.HandleMove.zone [ IMG_BAS_GAUCHE ] .width   =  9;
		view.HandleMove.zone [ IMG_BAS_GAUCHE ] .height  =  9;
		
		view.HandleMove.zone [ IMG_GAUCHE ] .x           = -4;
		view.HandleMove.zone [ IMG_GAUCHE ] .y           = (p_Image->image_height / 2) -4;
		view.HandleMove.zone [ IMG_GAUCHE ] .width       =  9;
		view.HandleMove.zone [ IMG_GAUCHE ] .height      =  9;
		
		view.HandleMove.zone [ IMG_MOVE ] .x             =  0;
		view.HandleMove.zone [ IMG_MOVE ] .y             =  0;
		view.HandleMove.zone [ IMG_MOVE ] .width         =  p_Image->image_width;
		view.HandleMove.zone [ IMG_MOVE ] .height        =  p_Image->image_height;
	}
}
void poche_set_rectangle( cairo_t *cr )
{
	gint	i;
	
	// GREEN COLOR
	cairo_set_source_rgb( cr, 0, 0.5, 0 );
	// SET RECTANGLES
	for( i = 0; i < IMG_SIZE -1; i ++ ) {
		cairo_rectangle( cr,
				 view.HandleMove.zone [ i ] .x,
				 view.HandleMove.zone [ i ] .y,
				 view.HandleMove.zone [ i ] .width,
				 view.HandleMove.zone [ i ] .height
				 );
	}
	cairo_fill( cr );
	// BLACK COLOR
	cairo_set_source_rgb( cr, 0, 0, 0 );
	// SET RECTANGLE AROUND
	cairo_rectangle( cr,
			 view.HandleMove.zone [ i ] .x,
			 view.HandleMove.zone [ i ] .y,
			 view.HandleMove.zone [ i ] .width,
			 view.HandleMove.zone [ i ] .height
			 );
}
// 
// 
gboolean on_drawingareamain_expose_event( GtkWidget *widget, GdkEventExpose *event, gpointer user_data )
{
	cairo_t	*cr;
	gint	canvas_w, canvas_h;
	GList	*List = NULL;
	IMAGE	*Image = NULL;
	
	view.scale = poche_get_scale_adjust();
	// gdk_drawable_get_size (view.AdrDrawingarea->window, &canvas_w, &canvas_h );
	canvas_w = view.AdrDrawingarea->allocation.width;
	canvas_h = view.AdrDrawingarea->allocation.height;
	view.x0 = ((canvas_w / view.scale) - view.image_width) / 2.0;
	view.y0 = ((canvas_h / view.scale) - view.image_height) / 2.0;
	cr = gdk_cairo_create( view.AdrDrawingarea->window );
	cairo_scale( cr, view.scale, view.scale );
	cairo_translate( cr, view.x0, view.y0 );
	cairo_set_source_surface( cr, view.image, 0.0, 0.0 );
	cairo_paint( cr );
	cairo_destroy( cr );
	
	cr = gdk_cairo_create( view.AdrDrawingarea->window );
	List = g_list_first( view.ListImage );
	while( List ) {
		if( NULL != (Image = (IMAGE *)List->data ) && TRUE == Image->BoolStructActived ) {
			cairo_save (cr);
			poche_draw_object( cr , Image );
			if( TRUE == Image->BoolIsSelected ) {
				cairo_set_source_rgb( cr, 0, 0, 0 );
				cairo_set_line_width( cr, 1 );
				cairo_translate( cr, 0.0, 0.0 );
				poche_get_handle_move( Image, FALSE );
				poche_set_rectangle( cr );
				cairo_stroke ( cr );
			}
			cairo_restore (cr);
		}
		List = g_list_next( List );
	}
	cairo_destroy( cr );
	poche_set_flag_buttons();
	return( FALSE );
}
// 
// 
IMAGE *poche_get_test( gdouble x, gdouble y )
{
	cairo_t	*cr;
	GdkRectangle	zone;
	GdkRegion	*region = NULL;
	IMAGE		*Image = NULL;
	GList		*List = NULL;
	gboolean	BoolInZone = FALSE;
	
	cr = gdk_cairo_create( view.AdrDrawingarea->window );
        cairo_scale (cr, view.scale, view.scale);
        cairo_translate (cr, view.x0, view.y0);
	List = g_list_last( view.ListImage );
	while( List ) {
		if( NULL != (Image = (IMAGE *)List->data ) && TRUE == Image->BoolStructActived ) {
			
			BoolInZone = FALSE;

			// IMG_MOVE
			zone.x      = Image->x0;
			zone.y      = Image->y0;
			zone.width  = Image->image_width;
			zone.height = Image->image_height;
			region = gdk_region_rectangle (&zone);
			if( gdk_region_point_in( region, x, y )) {
				BoolInZone = TRUE;
				Image->XPointer = x;
				Image->YPointer = y;
			}
			gdk_region_destroy (region);
			if( TRUE == BoolInZone ) {
				cairo_destroy( cr );
				return( Image );
			}
		}
		List = g_list_previous( List );
	}
	cairo_destroy( cr );
	return( NULL );
}
// 
// 
IMAGE *poche_get( gdouble x, gdouble y )
{
	cairo_t	*cr;
	// GdkRectangle	zone;
	GdkRegion	*region = NULL;
	IMAGE		*Image = NULL;
	GList		*List = NULL;
	gboolean	BoolInZone = FALSE;
	gint		CptImgCorner;
	
	cr = gdk_cairo_create( view.AdrDrawingarea->window );
        cairo_scale (cr, view.scale, view.scale);
        cairo_translate (cr, view.x0, view.y0);
	List = g_list_last( view.ListImage );
	while( List ) {
		if( NULL != (Image = (IMAGE *)List->data ) && TRUE == Image->BoolStructActived ) {
			
			if( TRUE != Image->BoolIsSelected ) {
				List = g_list_previous( List );
				continue;
			}
			
			BoolInZone = FALSE;
			
			poche_get_handle_move( Image, TRUE );
			for( CptImgCorner = 0; CptImgCorner < IMG_SIZE; CptImgCorner ++ ) {
				region = gdk_region_rectangle( &view.HandleMove.zone [ CptImgCorner ] );
				if( gdk_region_point_in( region, x, y )) {
					BoolInZone = TRUE;
					Image->XPointer = x;
					Image->YPointer = y;
				}
				gdk_region_destroy (region);
				if( TRUE == BoolInZone ) {
					cairo_destroy( cr );
					view.ImgCorner = CptImgCorner;
					switch( CptImgCorner ) {
					case IMG_HAUT_GAUCHE :	cursor_set_haut_gauche();	break;
					case IMG_HAUT :		cursor_set_haut();		break;
					case IMG_HAUT_DROIT :	cursor_set_haut_droit();	break;
					case IMG_DROIT :	cursor_set_droit();		break;
					case IMG_BAS_DROIT :	cursor_set_bas_droit();		break;
					case IMG_BAS :		cursor_set_bas();		break;
					case IMG_BAS_GAUCHE :	cursor_set_bas_gauche();	break;
					case IMG_GAUCHE :	cursor_set_gauche();		break;
					case IMG_MOVE :		cursor_set_move();		break;
					case IMG_SIZE :		
					case IMG_NONE :	
						cursor_set_old();
						break;
					}
					return( Image );
				}
			}
		}
		List = g_list_previous( List );
	}
	cursor_set_old();
	view.ImgCorner = IMG_NONE;
	cairo_destroy( cr );
	return( NULL );
}
// 
// 
void poche_set_selected( IMAGE *p_Image )
{
	GList	*List = NULL;
        IMAGE	*Image = NULL;
	GList	*ListLast = NULL;
        IMAGE	*ImageLast = NULL;
 	
	// GET List OF p_Image
	List = g_list_first( view.ListImage );
	while( List ) {
		if( NULL != (Image = (IMAGE *)List->data ) && TRUE == Image->BoolStructActived ) {
			if( p_Image == Image ) {
				break;
			}
		}
		List = g_list_next( List );
	}
	
	// GET ListLast
	ListLast = g_list_last( view.ListImage );
	while( ListLast ) {
		if( NULL != (ImageLast = (IMAGE *)ListLast->data ) && TRUE == Image->BoolStructActived ) {
			break;
		}
		ListLast = g_list_previous( ListLast );
	}
	
	// CHANGE List <-> ListLast
	if( NULL != List && NULL != ListLast  ) {
		ListLast->data = Image;
		List->data     = ImageLast;
		gtk_widget_queue_draw( view.AdrDrawingarea );
	}
}
// 
// 
gboolean on_drawingareamain_button_press_event( GtkWidget *widget, GdkEventButton *event, gpointer user_data )
{
	cairo_t	*cr;
        gdouble	x, y;
        guint	state;
        IMAGE	*Image = NULL;
        IMAGE	*ListImage = NULL;
	GList	*List = NULL;
        
	cr = gdk_cairo_create( view.AdrDrawingarea->window );
        cairo_scale (cr, view.scale, view.scale);
        cairo_translate (cr, view.x0, view.y0);
        x = event->x;
        y = event->y;
        cairo_device_to_user (cr, &x, &y);
	state = event->state;
	
	// IMAGE A DESSUS AVEC: CTRL + click
	if( state & GDK_CONTROL_MASK ) {
		if( IMG_NONE == view.ImgCorner || IMG_MOVE == view.ImgCorner ) {
			if( NULL != (Image = (IMAGE *)poche_get_test( x, y ))) {
				poche_set_selected( Image );
			}
		}
	}
	
	// NOUVELLE SELECTION D IMAGE
	if( IMG_NONE == view.ImgCorner || IMG_MOVE == view.ImgCorner ) {
		if( NULL != (Image = (IMAGE *)poche_get_test( x, y ))) {
			if( FALSE == Image->BoolIsSelected ) {
				List = g_list_first( view.ListImage );
				while( List ) {
					if( NULL != (ListImage = (IMAGE *)List->data ) && TRUE == Image->BoolStructActived ) {
						ListImage->BoolIsSelected = FALSE;
					}
					List = g_list_next( List );
				}
				Image->BoolIsSelected = TRUE;
				cursor_set_move();
				view.ImgCorner = IMG_MOVE;
				view.GetImage = Image;
				cairo_destroy( cr );
				gtk_widget_queue_draw( view.AdrDrawingarea );
				view.BoolEventButtonPress = TRUE;
				return( FALSE );
			}
		}
	}
	
	// DESELECTION
	if( IMG_NONE == view.ImgCorner && NULL == Image ) {
		List = g_list_first( view.ListImage );
		while( List ) {
			if( NULL != (Image = (IMAGE *)List->data ) && TRUE == Image->BoolStructActived ) {
				Image->BoolIsSelected = FALSE;
			}
			List = g_list_next( List );
		}
		cursor_set_old();
		cairo_destroy( cr );
		gtk_widget_queue_draw( view.AdrDrawingarea );
		view.BoolEventButtonPress = TRUE;
		return( FALSE );
	}
	
	cairo_destroy( cr );
	view.BoolEventButtonPress = TRUE;
	
	return( FALSE );
}
// 
// 
gboolean on_drawingareamain_button_release_event( GtkWidget *widget, GdkEventButton *event, gpointer user_data )
{
	cairo_t	*cr;
        gdouble	x, y;
        
	cr = gdk_cairo_create( view.AdrDrawingarea->window );
        cairo_scale (cr, view.scale, view.scale);
        cairo_translate (cr, view.x0, view.y0);
        x = event->x;
        y = event->y;
        cairo_device_to_user (cr, &x, &y);
	view.BoolEventButtonPress = FALSE;
	view.GetImage = NULL;
	cairo_destroy( cr );	
	return( FALSE );
}
// 
// 
gboolean on_drawingareamain_motion_notify_event( GtkWidget *widget, GdkEventMotion *event, gpointer user_data )
{
	cairo_t		*cr;
        gdouble		x, y;
	
	cr = gdk_cairo_create( view.AdrDrawingarea->window );
        cairo_scale (cr, view.scale, view.scale);
        cairo_translate (cr, view.x0, view.y0);
        x = event->x;
        y = event->y;
        cairo_device_to_user (cr, &x, &y);
	
	if (oldX == -1.0 || oldY == -1.0) {
		oldX = x;
		oldY = y;
	}
	
	if( FALSE == view.BoolEventButtonPress ) {
		view.GetImage = (IMAGE *)poche_get( x, y ); 
	}
	
	if( NULL != view.GetImage) {
		// g_print( "MOVE x = %f  y = %f\n", x , y );
	}
	
	if( TRUE == view.BoolEventButtonPress ) {
		if( NULL != view.GetImage ) {
			
			if( IMG_MOVE == view.ImgCorner ) {
				view.GetImage->x0       += (x - view.GetImage->XPointer);
				view.GetImage->y0       += (y - view.GetImage->YPointer);
				view.GetImage->XPointer  = x;
				view.GetImage->YPointer  = y;
				cairo_destroy( cr );
				gtk_widget_queue_draw( view.AdrDrawingarea );	
				oldX = x;
				oldY = y;
				return( FALSE );
			}
			else if( IMG_DROIT == view.ImgCorner ) {
				if( x > oldX ) {
					gint add = x - (view.GetImage->x0 + view.GetImage->image_width);
					view.GetImage->image_width += add;
				}
				else if( x < oldX ) {
					gint add = (view.GetImage->x0 + view.GetImage->image_width) - x;
					view.GetImage->image_width -= add;
				}
				if( x != oldX ) {
					cairo_destroy( cr );
					gtk_widget_queue_draw( view.AdrDrawingarea );	
					oldX = x;
					oldY = y;
					return( FALSE );
				}
			}
			else if( IMG_GAUCHE == view.ImgCorner ) {
				if( x > oldX ) {
					gint add = x - view.GetImage->x0;
					view.GetImage->x0 += add;
					view.GetImage->image_width -= add;
				}
				else if( x < oldX ) {
					gint add = view.GetImage->x0 - x;
					view.GetImage->x0 -= add;
					view.GetImage->image_width += add;
				}
				if( x != oldX ) {
					cairo_destroy( cr );
					gtk_widget_queue_draw( view.AdrDrawingarea );	
					oldX = x;
					oldY = y;
					return( FALSE );
				}
			}
			else if( IMG_HAUT == view.ImgCorner ) {
				if( y > oldY ) {
					gint add = y - view.GetImage->y0;
					view.GetImage->y0 += add;
					view.GetImage->image_height -= add;
				}
				else if( y < oldY ) {
					gint add = view.GetImage->y0 - y;
					view.GetImage->y0 -= add;
					view.GetImage->image_height += add;
				}
				if( y != oldY ) {
					cairo_destroy( cr );
					gtk_widget_queue_draw( view.AdrDrawingarea );	
					oldX = x;
					oldY = y;
					return( FALSE );
				}
			}
			else if( IMG_BAS == view.ImgCorner ) {
				if( y > oldY ) {
					gint add = y - (view.GetImage->y0 + view.GetImage->image_height);
					view.GetImage->image_height += add;
				}
				else if( y < oldY ) {
					gint add = (view.GetImage->y0 + view.GetImage->image_height) - y;
					view.GetImage->image_height -= add;
				}
				if( y != oldY ) {
					cairo_destroy( cr );
					gtk_widget_queue_draw( view.AdrDrawingarea );	
					oldX = x;
					oldY = y;
					return( FALSE );
				}
			}
			else if( IMG_HAUT_GAUCHE == view.ImgCorner ) {
				// HAUT
				if( y > oldY ) {
					gint add = y - view.GetImage->y0;
					view.GetImage->y0 += add;
					view.GetImage->image_height -= add;
				}
				else if( y < oldY ) {
					gint add = view.GetImage->y0 - y;
					view.GetImage->y0 -= add;
					view.GetImage->image_height += add;
				}
				// GAUCHE
				if( x > oldX ) {
					gint add = x - view.GetImage->x0;
					view.GetImage->x0 += add;
					view.GetImage->image_width -= add;
				}
				else if( x < oldX ) {
					gint add = view.GetImage->x0 - x;
					view.GetImage->x0 -= add;
					view.GetImage->image_width += add;
				}
				if( x != oldX || y != oldY ) {
					cairo_destroy( cr );
					gtk_widget_queue_draw( view.AdrDrawingarea );	
					oldX = x;
					oldY = y;
					return( FALSE );
				}
			}
			else if( IMG_HAUT_DROIT == view.ImgCorner ) {
				// HAUT
				if( y > oldY ) {
					gint add = y - view.GetImage->y0;
					view.GetImage->y0 += add;
					view.GetImage->image_height -= add;
				}
				else if( y < oldY ) {
					gint add = view.GetImage->y0 - y;
					view.GetImage->y0 -= add;
					view.GetImage->image_height += add;
				}
				// DROIT
				if( x > oldX ) {
					gint add = x - (view.GetImage->x0 + view.GetImage->image_width);
					view.GetImage->image_width += add;
				}
				else if( x < oldX ) {
					gint add = (view.GetImage->x0 + view.GetImage->image_width) - x;
					view.GetImage->image_width -= add;
				}
				if( x != oldX || y != oldY ) {
					cairo_destroy( cr );
					gtk_widget_queue_draw( view.AdrDrawingarea );	
					oldX = x;
					oldY = y;
					return( FALSE );
				}
			}
			else if( IMG_BAS_DROIT == view.ImgCorner ) {
				// BAS
				if( y > oldY ) {
					gint add = y - (view.GetImage->y0 + view.GetImage->image_height);
					view.GetImage->image_height += add;
				}
				else if( y < oldY ) {
					gint add = (view.GetImage->y0 + view.GetImage->image_height) - y;
					view.GetImage->image_height -= add;
				}
				// DROIT
				if( x > oldX ) {
					gint add = x - (view.GetImage->x0 + view.GetImage->image_width);
					view.GetImage->image_width += add;
				}
				else if( x < oldX ) {
					gint add = (view.GetImage->x0 + view.GetImage->image_width) - x;
					view.GetImage->image_width -= add;
				}
				if( x != oldX || y != oldY ) {
					cairo_destroy( cr );
					gtk_widget_queue_draw( view.AdrDrawingarea );	
					oldX = x;
					oldY = y;
					return( FALSE );
				}
			}
			else if( IMG_BAS_GAUCHE == view.ImgCorner ) {
				// BAS
				if( y > oldY ) {
					gint add = y - (view.GetImage->y0 + view.GetImage->image_height);
					view.GetImage->image_height += add;
				}
				else if( y < oldY ) {
					gint add = (view.GetImage->y0 + view.GetImage->image_height) - y;
					view.GetImage->image_height -= add;
				}
				// GAUCHE
				if( x > oldX ) {
					gint add = x - view.GetImage->x0;
					view.GetImage->x0 += add;
					view.GetImage->image_width -= add;
				}
				else if( x < oldX ) {
					gint add = view.GetImage->x0 - x;
					view.GetImage->x0 -= add;
					view.GetImage->image_width += add;
				}
				if( x != oldX || y != oldY ) {
					cairo_destroy( cr );
					gtk_widget_queue_draw( view.AdrDrawingarea );	
					oldX = x;
					oldY = y;
					return( FALSE );
				}
			}
		}
	}
	// g_print("event->x = %f   x = %f   oldX = %f\n", event->x, x, oldX );
	oldX = x;
	oldY = y;
	
	cairo_destroy( cr );
	return( FALSE );
}
// 
// 
void on_button_moins_clicked( GtkButton *button, gpointer user_data )
{
	view.BoolScaleAdjust = FALSE;
	view.scale -= 0.5;
	if( view.scale < 0.4 ) view.scale = 0.4;
	poche_set_size_request();
	gtk_widget_queue_draw( view.AdrDrawingarea );
}
// 
// 
void on_button_normal_clicked( GtkButton *button, gpointer user_data )
{
	view.BoolScaleAdjust = FALSE;
	view.scale  = 1.0;	
	poche_set_size_request();
	gtk_widget_queue_draw( view.AdrDrawingarea );
}
// 
// 
void on_button_plus_clicked( GtkButton *button, gpointer user_data )
{
	view.BoolScaleAdjust = FALSE;
	view.scale += 0.5;
	if( view.scale > 8.0 ) view.scale = 8.0;
	poche_set_size_request();
	gtk_widget_queue_draw( view.AdrDrawingarea );
}
// 
// 
void on_button_ajuster_clicked( GtkButton *button, gpointer user_data )
{
	view.BoolScaleAdjust = TRUE;
	gtk_widget_queue_draw( view.AdrDrawingarea );
}
// 
// 
IMAGE *poche_add_to_glist( gchar *PathNameFile, gdouble x, gdouble y, gboolean p_BoolScale, gboolean p_BoolSelected )
{
	IMAGE	*New = (IMAGE *)g_malloc0( sizeof(IMAGE) );
	
	New->PixbufOriginal = gdk_pixbuf_new_from_file( PathNameFile, NULL );
	if( TRUE == p_BoolScale )
		New->Pixbuf = gdk_pixbuf_scale_simple( New->PixbufOriginal, 100, 100, 1 );	// GDK_INTERP_NEAREST
	else	New->Pixbuf = NULL;
	New->image          = cairo_image_surface_create_from_png( PathNameFile );

	// New->image_width    = cairo_image_surface_get_width( New->image );
	// New->image_height   = cairo_image_surface_get_height( New->image );
	if( NULL != New->Pixbuf ) {
		New->image_width    = gdk_pixbuf_get_width( New->Pixbuf );
		New->image_height   = gdk_pixbuf_get_height( New->Pixbuf );
	}
	else {
		New->image_width    = gdk_pixbuf_get_width( New->PixbufOriginal );
		New->image_height   = gdk_pixbuf_get_height( New->PixbufOriginal );
	}

	New->x0                = x - (New->image_width / 2);
	New->y0                = y - (New->image_height / 2);
	
	New->BoolIsSelected    = p_BoolSelected;
	New->BoolStructActived = TRUE;

	view.ListImage = g_list_append( view.ListImage, New );
	
	return( New );
}
// 
// 
static void poche_drag_data_received (GtkWidget	        *widget,
				     GdkDragContext     *drag_context,
				     gint                x,
				     gint                y,
				     GtkSelectionData   *data,
				     guint               info,
				     guint               time)
{
	// PRINT("DND FROM DESKTOP TO DRAWING"); 
	// Une copie ne peut aller vers elle meme !!!
	if (gtk_drag_get_source_widget(drag_context) != widget) {
		dragndrop_list_drag_data (widget, (gchar *)data->data);
	}
}
// 
// 
void on_drawingareamain_realize( GtkWidget *widget, gpointer user_data )
{
	gchar	*PathNamefilePreview = NULL;
	gchar	*PathNamefilePreviewCurrentDir = NULL;
	gchar	*CurrentDir = g_get_current_dir ();
	gboolean	BoolExitFailure = FALSE;
	
	view.AdrDrawingarea  = widget;
	view.BoolScaleAdjust = FALSE;
	view.scale           = 1.0;
	
	PathNamefilePreviewCurrentDir = g_strdup_printf ("%s/pixmaps/preview.png", CurrentDir);
	PathNamefilePreview           = g_strdup_printf ("%s/pixmaps/zoomc/preview.png", DATA_DIR );
							
	if( TRUE == g_file_test( PathNamefilePreview, G_FILE_TEST_EXISTS )) {
		view.image           = cairo_image_surface_create_from_png( PathNamefilePreview );
		g_print( "LOAD: %s\n", PathNamefilePreview );
	}
	else if( TRUE == g_file_test( PathNamefilePreviewCurrentDir, G_FILE_TEST_EXISTS )) {
		view.image           = cairo_image_surface_create_from_png( PathNamefilePreviewCurrentDir );
		g_print( "LOAD: %s\n", PathNamefilePreviewCurrentDir );
	}
	else {
		BoolExitFailure = TRUE;
	}
	
	if( TRUE == BoolExitFailure ) {
		g_print( "LE FICHIER preview.png EST ABSENT !!!\n\n");
		g_print( "PathNamefilePreviewCurrentDir = %s\n", PathNamefilePreviewCurrentDir );
		g_print( "PathNamefilePreview           = %s\n", PathNamefilePreview );
	}
	g_free( PathNamefilePreviewCurrentDir );
	PathNamefilePreviewCurrentDir = NULL;
	g_free( PathNamefilePreview );
	PathNamefilePreview = NULL;
	g_free( CurrentDir );
	CurrentDir = NULL;
	
	if( TRUE == BoolExitFailure ) {
		exit( EXIT_FAILURE );
	}
	
	view.image_width     = cairo_image_surface_get_width( view.image );
	view.image_height    = cairo_image_surface_get_height( view.image );
	
	// poche_add_to_glist( "/home/cat/tool.png", TRUE, FALSE );
	// poche_add_to_glist( "/home/cat/42.png", TRUE, FALSE );
	// poche_add_to_glist( "/home/cat/basebuild.png", TRUE, TRUE );
	
	poche_set_size_request();
	
	// Drag and drop support
	// SIGNAL : 'drag-data-received'
	gtk_drag_dest_set (GTK_WIDGET (widget),
			   GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP | GTK_DEST_DEFAULT_HIGHLIGHT,
			   drag_types,
			   n_drag_types,
			   GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_DEFAULT
			   );
	g_signal_connect(G_OBJECT(widget),
			 "drag-data-received",
			 G_CALLBACK(poche_drag_data_received),
			 NULL);

}
// 
// 
void poche_gestion_images (gchar *path)
{
	g_print( "path = %s\n", path );
}
// 
// 
void on_button_load_file_clicked (GtkButton *button, gpointer user_data)
{
	fileselect_create( _PATH_IMPORT_IMAGES_, getenv ("HOME"), poche_gestion_images );
}
// 
// 
void on_button_save_clicked (GtkButton *button, gpointer user_data)
{
	PRINT_FUNC_LF();
}
// 
// Cette fonction suppose un flag de (non) validite d'une structure IMAGE
// 
void on_button_erase_clicked (GtkButton *button, gpointer user_data)
{
	GList	*List = NULL;
        IMAGE	*Image = NULL;
        
	List = g_list_first( view.ListImage );
	while( List ) {
		if( NULL != (Image = (IMAGE *)List->data ) && TRUE == Image->BoolStructActived ) {
			if( TRUE == Image->BoolIsSelected ) {
				Image->BoolStructActived = FALSE;
				gtk_widget_queue_draw( view.AdrDrawingarea );
			}
		}
		List = g_list_next( List );
	}
}
// 
// 
static void poche_drag_data_received_file (GtkWidget          *widget,
				     		GdkDragContext     *drag_context,
				     		gint                x,
				     		gint                y,
				    		GtkSelectionData   *data,
				    		guint               info,
				    		guint               time)
{
	dragndrop_list_drag_data (widget, (gchar *)data->data);
}
// 
// 
void poche_table_add_images_init (void)
{
	if( NULL != view.Adr_table ) {
		gtk_widget_destroy( view.Adr_table );
		view.Adr_table = NULL;
	}
	view.Adr_table = gtk_table_new (4, 5, TRUE);
	gtk_container_add (GTK_CONTAINER (view.Adr_viewport_image_preview), view.Adr_table);
	gtk_widget_show_all (view.Adr_table);
}
// 
// 
void on_viewport_stock_realize (GtkWidget *widget, gpointer user_data)
{
	view.Adr_viewport_image_preview = widget;
	view.Adr_table = NULL;
	poche_table_add_images_init ();

	// Drag and drop support
	// SIGNAL : 'drag-data-received'
	// 
	gtk_drag_dest_set (GTK_WIDGET (widget),
			   GTK_DEST_DEFAULT_MOTION |
			   GTK_DEST_DEFAULT_DROP,
			   drag_types, n_drag_types,
			   GDK_ACTION_COPY| GDK_ACTION_MOVE );
	g_signal_connect(G_OBJECT(widget),
			 "drag-data-received",
			 G_CALLBACK(poche_drag_data_received_file),
			 NULL);
}
// 
// 
void poche_set_selected_image( IMAGE *p_Image )
{
	GList	*List = NULL;
        IMAGE	*Image = NULL;
 	
	// GET List OF p_Image to FALSE
	List = g_list_first( view.ListImage );
	while( List ) {
		if( NULL != (Image = (IMAGE *)List->data ) && TRUE == Image->BoolStructActived ) {
			Image->BoolIsSelected = FALSE;
		}
		List = g_list_next( List );
	}
	
	// SET p_Image to TRUE
	p_Image->BoolIsSelected = TRUE;
}
// 
// 
void poche_add_img_file (GSList *p_list)
{
	GSList		*gs_List = p_list;
	gchar		*Ptr = NULL;
	IMAGE		*Image = NULL;
	gdouble		x, y;
	gint		X, Y;
	GdkModifierType	state;
	cairo_t		*cr;

	gdk_window_get_pointer( view.AdrDrawingarea->window, &X, &Y, &state ); 	
	cr = gdk_cairo_create( view.AdrDrawingarea->window );
        cairo_scale (cr, view.scale, view.scale);
        cairo_translate (cr, view.x0, view.y0);
        x = X;
        y = Y;
        cairo_device_to_user (cr, &x, &y);
	cairo_destroy( cr );
	
	while (gs_List) {
		if ((Ptr = (gchar *)gs_List->data) != NULL) {
			break;
		}
		gs_List = g_slist_next (gs_List);
	}

	if( TRUE != FileIs_png( Ptr ) && TRUE != FileIs_jpg( Ptr )) {
		PRINT("BAD FILE");
		g_print( "\t%s\n", Ptr );
		return;
	}
	
	Image = poche_add_to_glist( Ptr, x, y, TRUE, FALSE );
	poche_set_selected_image( Image );
	gtk_widget_queue_draw( view.AdrDrawingarea );
}
// 
// 
gboolean on_windmain_key_press_event( GtkWidget *widget, GdkEventKey *kevent, gpointer user_data )
{
	if( kevent->keyval == GDK_Delete) {
		PRINT("GDK_Delete");
		on_button_erase_clicked( NULL, NULL );
		
	}
	return FALSE;
}
// 
// 
gboolean on_windmain_key_release_event( GtkWidget *widget, GdkEventKey *kevent, gpointer user_data )
{
	return FALSE;
}













